@@ -1,4 +1,4 @@
-# $Id: Debget.pm,v 1.1 2005/07/06 13:14:36 roderick Exp $
+# $Id: Debget.pm,v 1.2 2005/07/11 21:22:35 roderick Exp $
 #
 # Roderick Schertler <roderick@argon.org>
 
@@ -34,7 +34,7 @@ use them, too.  I will do everything possible to retain backwards
 compatibility for the interfaces documented here.
 
 Most of these functions will C<die> if they're called in a context
-(scalar, list) which doesn't match what they return.  This is so that I
+(scalar, list) which doesn't match what they expect.  This is so that I
 will be able to change them to return something different in the other
 context without breaking old code.
 
@@ -48,7 +48,7 @@ use Carp		  ();
 use List::Util		qw(max);
 use Memoize		qw(memoize);
 
-our $VERSION = q$Revision: 1.1 $ =~ /(\d\S+)/ ? $1 : '?';
+our $VERSION = q$Revision: 1.2 $ =~ /(\d\S+)/ ? $1 : '?';
 
 our (@EXPORT, @EXPORT_OK);
 
@@ -68,6 +68,7 @@ BEGIN {
     @EXPORT_OK	= qw(
 	$Debug
 	$Debug_fh
+	$Exit
 	$Me
 	debug
 	debug_hash
@@ -78,6 +79,7 @@ BEGIN {
 	xconfess
 	xcroak
 	xdie
+	xwarn
     );
 }
 
@@ -95,6 +97,7 @@ our $Pdo_re_no_package = qr/Can't find that package./;
 (our $Me		= $0) =~ s|.*/||;
 our $Debug		||= 0;
 our $Debug_fh		||= \*STDERR;
+our $Exit		||= 0;
 
 sub dstr ($) {
     local $_ = shift;
@@ -136,13 +139,15 @@ sub debug_hash {
 
 sub die_prefix {
     (my $program = $0) =~ s|.*/||;
-    my $sub = (caller 2)[3];
-    return "$program: $sub: ";
+    my @c = caller 2;
+    my $sub = $c[0] eq 'main' ? '' : "$c[3]: ";
+    return "$program: $sub";
 }
 
 sub xconfess	{ Carp::confess	die_prefix, @_ }
 sub xcroak	{ Carp::croak	die_prefix, @_ }
 sub xdie	{ die		die_prefix, @_ }
+sub xwarn	{ warn		die_prefix, @_; $Exit ||= 1 }
 
 # This is basically Getopt::Long but it has the defaults set up the way I
 # think they should be.
@@ -181,6 +186,8 @@ sub getopt {
     goto &Getopt::Long::GetOptions;
 }
 
+#------------------------------------------------------------------------------
+
 =head1 IMPORTABLES
 
 =over 4
@@ -243,27 +250,23 @@ sub get_uri {
     return $resp;
 } }
 
-#    # XXX describe failure, link to article on Google
+# My current method of finding what version of a package is to parse a
+# http://packages.debian.org query.
+
+# This lacks some info I'd like
+# (source for a binary package, component of either kind of package).  I'd
 #
-#    if (0) {
-#	require HTML::TreeBuilder;
+# also using xml
 #
-#	my $tree = HTML::TreeBuilder->new_from_content(\$content)
-#	    or die;
-#	$tree->objectify_text;
+# mailed 1 guy, mail the other
+
+# Originally I tried parsing the packages.debian.org data using
+# HTML::TreeBuilder, but I ran into a problem described in
 #
-#	my $eh = $tree->find('h1')
-#	    or xdie $p, "no <h1> tag";
-#	#$eh->dump;
-#	$eh->parent->dump;
+#    news:87oea1gp9l.fsf@eeyore.ibcinc.com
+#    http://groups-beta.google.com/group/comp.lang.perl.modules/msg/b7c72820e4895b42
 #
-#	my $ep = $eh->find('p')
-#	    or xdie $p, "no <p> tag after <h1>";
-#	$ep->dump;
-#
-#	$tree->delete;
-#	die;
-#    }
+# I also sent it to Sean Burke, but haven't heard back.
 
 sub parse_pdo_binary {
     @_ == 2
@@ -422,6 +425,7 @@ sub package_info_helper {
     debug "type=$type name=$name";
 
     defined $name
+	# XXX shows memoize location
 	or xcroak "undefined name";
     $name =~ /^\S+\z/
 	or xcroak "invalid name `$name'";
@@ -481,6 +485,16 @@ sub package_source {
 
     my $r = $apt_packages->lookup($pname);
     if (!$r) {
+    	# It wasn't found in the packages cache.  It might not exist,
+	# or it might be a source package name, or it might be either
+	# source or binary but not in the cache.
+	if (source_package_info $pname) {
+	    # it's a source package
+    	    debug "$pname -> $pname (via pdo)";
+    	    return $pname;
+    	}
+	# XXX a binary package name not found in the cache fails, need
+    	# to extend pdo to tell you the source package name for a binary
 	debug "$pname -> not found";
 	return;
     }
